import React from 'react'
import styled from 'styled-components'
import { Icon } from '@pubsweet/ui'
import { th } from '@pubsweet/ui-toolkit'

const NoteEvent = styled.div`
  display: flex;
  flex-wrap: wrap;
  justify-content: space-between;
`

const NoteTo = styled.div`
  font-weight: bold;
  width: 100%;
`

const NoteSubject = styled.div`
  font-weight: bold;
  width: 90%;
`

const NoteBodyControl = styled.div`
  width: 10%;
  text-align: right;
  align-self: flex-end;
`

const MetaContent = styled.span`
  font-weight: normal;
`

const EmailBody = styled.div`
  white-space: pre-wrap;
  font-style: italic;
  width: 100%;
  padding-top: calc(${th('gridUnit')} * 2);
`

const capitalizeFirstLetter = string =>
  string.charAt(0).toUpperCase() + string.slice(1)

const toWho = (manuscript, userId) => {
  const who = manuscript.teams.reduce((who, team) => {
    if (
      team.teamMembers &&
      team.teamMembers.length > 0 &&
      team.teamMembers[0].user &&
      userId.includes(team.teamMembers[0].user.id) &&
      team.teamMembers[0].alias &&
      team.teamMembers[0].alias.name
    ) {
      const { name } = team.teamMembers[0].alias
      who = `${who && `${who}, `} ${capitalizeFirstLetter(team.role)} ${
        name.givenNames
      } ${name.surname}`
    }
    return who
  }, '')
  return who
}

const toggleHidden = elements => {
  elements.forEach(element => {
    element.classList.toggle('hidden')
  })
}

export const obtainEventDescription = (audit, manuscript) => {
  const { originalData, objectType, changes, user } = audit
  if (objectType === 'manuscript') {
    if (changes.status) {
      return `Manuscript ${changes.status}`
    }
    if (changes.journal_id) {
      return `Manuscript added journal information: ${
        manuscript.journal.journalTitle
      }`
    }
    if (changes.claimed_by) {
      let claimedBy = ''
      if (manuscript.claimedBy) {
        claimedBy = `${manuscript.claimedBy.givenNames} ${
          manuscript.claimedBy.surname
        }`
      }
      return `Manuscript claimed by ${claimedBy}`
    }
    if (changes['meta,release_delay']) {
      return `Manuscript added embargo period ${
        changes['meta,release_delay']
      } months`
    }
    if (changes['meta,funding_group']) {
      const funding = JSON.parse(changes['meta,funding_group'])
        .map(funding => funding.title)
        .join(', ')
      return `Manuscript added funding ${funding}`
    }
  }

  if (objectType === 'note') {
    const content = JSON.parse(changes.content)
    if (content && content.to) {
      return (
        <NoteEvent>
          <NoteTo>
            To: <MetaContent>{toWho(manuscript, content.to)}</MetaContent>
          </NoteTo>
          <NoteSubject>
            Subject: <MetaContent>{content.subject}</MetaContent>
          </NoteSubject>
          <NoteBodyControl
            onClick={e => toggleHidden([e.currentTarget.nextElementSibling])}
          >
            <Icon
              color="currentColor"
              onClick={e =>
                toggleHidden([
                  e.currentTarget,
                  e.currentTarget.nextElementSibling,
                ])
              }
              size={3}
            >
              chevron-right
            </Icon>
            <Icon
              className="hidden"
              color="currentColor"
              onClick={e =>
                toggleHidden([
                  e.currentTarget,
                  e.currentTarget.previousElementSibling,
                ])
              }
              size={3}
            >
              chevron-down
            </Icon>
          </NoteBodyControl>
          <EmailBody className="hidden">
            <hr />
            {content.message}
          </EmailBody>
        </NoteEvent>
      )
    }

    return `Note: ${content}`
  }

  if (objectType === 'team') {
    if (changes.role_name === 'submitter') {
      return `Manuscript submitted by ${user.givenNames} ${user.surname}`
    }
    if (changes.role_name === 'reviewer') {
      return `Manuscript added reviewer`
    }
  }

  if (objectType === 'file') {
    if (changes.type === 'manuscript') {
      return `Manuscript file uploaded: ${changes.filename}`
    }
    if (changes.filename) {
      return `File ${changes.filename} of mime type ${
        changes.mime_type
      } uploaded`
    }
    if (changes.type) {
      return `File ${originalData.filename}'s type set to ${changes.type}`
    }
    if (changes.label) {
      return `File ${originalData.filename}'s label set to ${changes.label}`
    }
  }

  return ''
}
