import React from 'react'
import { Query } from 'react-apollo'
import styled from 'styled-components'
import ReactHtmlParser from 'react-html-parser'
import { th, lighten } from '@pubsweet/ui-toolkit'
import { Action, Button, ErrorText, H1, H2 } from '@pubsweet/ui'
import {
  Loading,
  LoadingIcon,
  Buttons,
  Center,
  PreviewPage,
  PreviewPanel,
  EditPanel,
  PanelHeader,
  PanelContent,
} from '../ui'
import Annotator from '../Annotator'
import UploadFiles, { SubmissionTypes, ReviewTypes } from '../upload-files'
import ManuscriptPreview from '../ManuscriptPreview'
import PDFViewer from '../pdf-viewer'
import blueXSD from '../../assets/xsl/xsd/publishing/journalpublishing3.xsd'
import greenXSD from '../../assets/xsl/xsd/archiving/archivearticle3.xsd'
import { GET_MANUSCRIPT } from '../operations'
import SubmissionHeader from '../SubmissionHeader'
import HTMLPreview from './HTMLPreview'
import ReviewForm from './ReviewForm'

const PreviewPageDiv = styled(PreviewPage)`
  & > div > div {
    overflow: auto;
  }
  & > div > div > div:last-child {
    height: calc(100% - (${th('gridUnit')} * 19));
  }
  .show-mobile {
    display: none;
  }
  .pad {
    padding: 0 calc(2 * ${th('gridUnit')});
  }
  @media screen and (max-width: 870px) {
    .show-mobile {
      display: inline-block;
    }
    .hide-mobile {
      display: none;
    }
  }
`
const PreviewPanelHeader = styled(PanelHeader)`
  height: calc(${th('gridUnit')} * 10);

  @media screen and (max-width: 870px) {
    height: auto;
  }
`
const Toggle = styled.div`
  height: calc(${th('gridUnit')} * 7);
  button {
    vertical-align: top;
    font-weight: 600;
  }
  button + button {
    margin-left: calc(${th('gridUnit')} * 6);
  }
  &.hide-mobile button {
    margin-left: calc(${th('gridUnit')} * 3);
  }
  @media screen and (max-width: 870px) {
    height: auto;
    margin-bottom: calc(${th('gridUnit')} * 3);
    display: flex;
    align-items: center;
    justify-content: space-between;
    flex-wrap: wrap;
    button {
      padding: ${th('gridUnit')} ${th('gridUnit')} 0;
    }
    button + button {
      margin: 0;
    }
  }
`
const PreviewError = styled.div`
  border: ${th('borderWidth')} ${th('borderStyle')} ${th('colorError')};
  background-color: ${lighten('colorError', 40)};
  padding: ${th('gridUnit')} calc(${th('gridUnit')} * 2);
  margin: ${th('gridUnit')} 0;
`
const AnnotatePDF = Annotator(PDFViewer)
const AnnotateHTML = Annotator(HTMLPreview)

const allFileTypes = ReviewTypes.concat(
  SubmissionTypes.reduce((orgf, sub) => {
    if (!ReviewTypes.some(existing => existing.value === sub.value)) {
      orgf.push(sub)
    }
    return orgf
  }, []),
)

const transformXML = (manuscript, xmlUri, xsdBUri, xsdGUri, param) =>
  new Promise((resolve, reject) => {
    fetch('/xml/validate/api/post', {
      method: 'POST',
      headers: {
        Accept: 'text/json',
        Authorization: `Bearer ${window.localStorage.getItem('token')}`,
        'Content-Type': 'application/json',
      },
      body: JSON.stringify({
        manuscript,
        xmlUri,
        xsdBUri,
        xsdGUri,
        param,
      }),
    })
      .then(response => {
        if (!response.ok) {
          reject(response.text())
        } else {
          resolve(response.text())
        }
      })
      .catch(reason => reject(reason))
  })

const getAnnotations = async id => {
  const response = await fetch(`/annotations/file/${id}`, {
    headers: new Headers({
      Authorization: `Bearer ${window.localStorage.getItem('token')}`,
    }),
  })
  const json = await response.json()
  return json.rows
}

class Review extends React.Component {
  constructor(props) {
    super(props)
    this.state = {
      pane: this.props.currentUser.admin ? 'files' : 'web',
      error: '',
      showManuscript: false,
      showAll: false,
      html: null,
      annotations: null,
    }
    this.listAnnotations = this.listAnnotations.bind(this)
    this.processXML = this.processXML.bind(this)
    this.webPreview = this.webPreview.bind(this)
    this.setError = this.setError.bind(this)
  }
  componentDidMount() {
    this.listAnnotations()
  }
  async listAnnotations() {
    const { manuscript } = this.props
    const annotations = []
    annotations.push({
      name: 'Web',
      value: await getAnnotations(`${manuscript.id}-web`),
    })
    annotations.push({
      name: 'PDF',
      value: await getAnnotations(`${manuscript.id}-pdf`),
    })
    this.setState({ annotations })
  }
  setError(error) {
    // replace with note mutation
    this.setState({ error })
  }
  async processXML() {
    const { manuscript } = this.props
    const { files } = manuscript
    const xml = files.find(file => file.type === 'PMC')
    if (xml) {
      transformXML(manuscript, xml.url, blueXSD, greenXSD, files)
        .then(result => {
          this.setError('')
          this.props.addXMLFiles(JSON.parse(result), manuscript)
        })
        .catch(async err => {
          this.setError(await err)
        })
    }
  }
  async webPreview() {
    const { manuscript } = this.props
    const { files } = manuscript
    const html = files.find(file => file.type === 'tempHTML')
    if (html) {
      const response = await fetch(html.url)
      this.setState({ html: await response.text() })
    } else {
      this.setState({ html: 'Error. No Web Preview generated.' })
    }
  }
  render() {
    const { manuscript, currentUser } = this.props
    const { files: allfiles, meta } = manuscript
    const sourceFile = allfiles.find(file => file.type === 'manuscript')
    const files = allfiles.filter(
      file => !file.type || ReviewTypes.some(rev => rev.value === file.type),
    )
    /* const originalFiles = allfiles.filter(
      file => !file.type || SubmissionTypes.some(sub => sub.value === file.type),
    ) */
    const pdf = files.find(
      file => file.type === 'pdf4load' || file.type === 'pdf4print',
    )
    const xmlError = meta.notes
      ? meta.notes.find(n => n.notesType === 'xml error')
      : ''
    const { pane, error, showManuscript, showAll, html } = this.state
    return (
      <PreviewPageDiv>
        <PreviewPanel
          style={{
            flex: showManuscript && '1 1 750px',
            width: showManuscript && '50%',
          }}
        >
          <div style={{ maxWidth: showManuscript && '750px' }}>
            <PreviewPanelHeader>
              <H1>Review web versions of manuscript</H1>
            </PreviewPanelHeader>
            <Toggle>
              {currentUser.admin && (
                <Action
                  className={pane === 'files' ? 'current' : ''}
                  onClick={() => this.setState({ pane: 'files' })}
                >
                  XML Files
                </Action>
              )}
              <Action
                className={pane === 'web' ? 'current' : ''}
                onClick={() => {
                  this.webPreview()
                  this.setState({ pane: 'web' })
                }}
              >
                Web Preview
              </Action>
              <Action
                className={pane === 'pdf' ? 'current' : ''}
                onClick={() => this.setState({ pane: 'pdf' })}
              >
                PDF Preview
              </Action>
              <Action
                className={
                  pane === 'original' ? 'current show-mobile' : 'show-mobile'
                }
                onClick={() => this.setState({ pane: 'original' })}
                primary={pane !== 'original'}
              >
                Submitted Manuscript
              </Action>
            </Toggle>
            <PanelContent>
              {pane === 'files' && (
                <React.Fragment>
                  <Buttons left top>
                    <Button
                      onClick={this.processXML}
                      style={{ display: 'inline-flex', alignItems: 'center' }}
                    >
                      Regenerate previews
                    </Button>
                    <Button
                      onClick={() => this.setState({ showAll: !showAll })}
                      style={{ display: 'inline-flex', alignItems: 'center' }}
                    >
                      {`${showAll ? 'Hide' : 'Show'} submission files`}
                    </Button>
                  </Buttons>
                  {xmlError && (
                    <PreviewError>
                      {ReactHtmlParser(xmlError.content)}
                    </PreviewError>
                  )}
                  {error && (
                    <Center>
                      <ErrorText>{error}</ErrorText>
                    </Center>
                  )}
                  <UploadFiles
                    checked
                    files={showAll ? allfiles : files}
                    manuscript={manuscript.id}
                    types={showAll ? allFileTypes : ReviewTypes}
                  />
                  {error && (
                    <Center>
                      <ErrorText>{error}</ErrorText>
                    </Center>
                  )}
                  <Buttons left top>
                    <Button
                      onClick={this.processXML}
                      style={{ display: 'inline-flex', alignItems: 'center' }}
                    >
                      Regenerate previews
                    </Button>
                    <Button
                      onClick={() => this.setState({ showAll: !showAll })}
                      style={{ display: 'inline-flex', alignItems: 'center' }}
                    >
                      {`${showAll ? 'Hide' : 'Show'} submission files`}
                    </Button>
                  </Buttons>
                </React.Fragment>
              )}
              {pane === 'pdf' && pdf && (
                <AnnotatePDF
                  reload={this.listAnnotations()}
                  toMark={`${manuscript.id}-pdf`}
                  url={pdf.url}
                  user={currentUser.id}
                />
              )}
              {pane === 'web' && html && (
                <AnnotateHTML
                  html={html}
                  reload={this.listAnnotations()}
                  toMark={`${manuscript.id}-web`}
                  user={currentUser.id}
                />
              )}
              {pane === 'original' && sourceFile && (
                <ManuscriptPreview file={sourceFile} />
              )}
            </PanelContent>
          </div>
        </PreviewPanel>
        <EditPanel
          style={{
            flex: showManuscript && '1 1 750px',
            width: showManuscript && '50%',
          }}
        >
          <div style={{ maxWidth: showManuscript && '750px' }}>
            <PreviewPanelHeader>
              <H2>Compare &amp; approve</H2>
            </PreviewPanelHeader>
            <Toggle className="hide-mobile">
              <Button
                onClick={() =>
                  this.setState({ showManuscript: !showManuscript })
                }
              >
                {showManuscript
                  ? 'Back to form'
                  : 'Compare with submitted file'}
              </Button>
            </Toggle>
            <PanelContent className={showManuscript && 'pad'}>
              {showManuscript && sourceFile ? (
                <ManuscriptPreview file={sourceFile} />
              ) : (
                <ReviewForm
                  annotations={this.state.annotations}
                  manuscript={manuscript}
                />
              )}
            </PanelContent>
          </div>
        </EditPanel>
      </PreviewPageDiv>
    )
  }
}

const ReviewWithHeader = SubmissionHeader(Review)

const ReviewPage = ({ match, ...props }) => (
  <Query
    fetchPolicy="cache-and-network"
    query={GET_MANUSCRIPT}
    variables={{ id: match.params.id }}
  >
    {({ data, loading }) => {
      if (loading || !data || !data.manuscript) {
        return (
          <Loading>
            <LoadingIcon />
          </Loading>
        )
      }
      return (
        <ReviewWithHeader
          manuscript={data.manuscript}
          match={match}
          {...props}
        />
      )
    }}
  </Query>
)

export default ReviewPage
