const path = require('path')
const mime = require('mime-types')
const FormData = require('form-data')
const passport = require('passport')
const libxml = require('libxmljs')
const config = require('config')
const fetch = require('node-fetch')
const xslt4node = require('xslt4node')

xslt4node.addLibrary('./saxon9he.jar')
xslt4node.addOptions('-Xmx1g')

const pubsweetServer = config.get('pubsweet-server.baseUrl')
const authBearer = passport.authenticate('bearer', { session: false })

const fetchFile = (fileUri, token) => {
  const url = `${pubsweetServer}${fileUri}`
  return new Promise((resolve, reject) => {
    fetch(url, {
      method: 'GET',
      headers: {
        Authorization: `${token}`,
        Accept: 'application/xml',
      },
    })
      .then(data => {
        resolve(data.text())
      })
      .catch(err => {
        reject(err)
      })
  })
}

const transformXML = (xmlString, xslPath, params) => {
  xmlString = xmlString.replace(/<!DOCTYPE[^>[]*(\\[[^]]*\\])?>/, '')
  const config = {
    xsltPath: xslPath,
    source: xmlString,
    result: String,
    props: {
      indent: 'yes',
    },
  }
  if (params) {
    config.params = params
  }
  return new Promise((resolve, reject) => {
    xslt4node.transform(config, (err, result) => {
      if (err) {
        reject(result)
      } else {
        resolve(result)
      }
    })
  })
}

const pushBlob = (blob, filename, token) => {
  const form = new FormData()
  form.append('file', blob, filename)
  let mimeType = mime.contentType(path.extname(filename))
  if (!mimeType) mimeType = 'application/xml' // default
  const url = `${pubsweetServer}/api/upload`
  return new Promise((resolve, reject) => {
    fetch(url, {
      method: 'POST',
      headers: {
        Accept: 'text/plain',
        Authorization: `Bearer ${token}`,
      },
      body: form,
    })
      .then(response => response.text())
      .then(data =>
        resolve({
          fileURI: data,
          filename,
          fileSize: blob.length,
          mimeType,
        }),
      )
      .catch(err => {
        reject(err)
      })
  })
}

const addXMLFiles = async (nxml, html, project, currentVersion, token) => {
  const { files } = currentVersion
  const newFiles = files.filter(
    file => file.type !== 'PMCfinal' && file.type !== 'tempHTML',
  )
  // TODO: Change filename to use ID!
  const upXML = await pushBlob(nxml, 'converted.nxml', token)
  newFiles.push({
    filename: upXML.filename,
    mimeType: upXML.mimeType,
    size: upXML.fileSize,
    url: upXML.fileURI,
    type: 'PMCfinal',
  })
  const upHTML = await pushBlob(html, 'converted.html', token)
  newFiles.push({
    filename: upHTML.filename,
    mimeType: upHTML.mimeType,
    size: upHTML.fileSize,
    url: upHTML.fileURI,
    type: 'tempHTML',
  })
  return newFiles
}

function getFilelist(files) {
  return {
    filelist: files
      .filter(
        file =>
          file.type === 'IMGview' ||
          file.type === 'supplement_tag' ||
          file.type === 'supplementary file',
      )
      .map(
        file =>
          `${file.filename.substring(
            0,
            file.filename.lastIndexOf('.'),
          )}:/api/files/${file.url.split('/').pop()};`,
      )
      .join(''),
  }
}

module.exports = app => {
  app.post('/xml/validate/api/post', authBearer, (req, res) => {
    res.set({ 'Content-Type': 'text/html' })
    // fetch the submitted xml
    fetchFile(req.body.xmlUri, req.headers.authorization).then(xml => {
      // fetch XSDs
      fetchFile(req.body.xsdBUri, req.headers.authorization).then(xsdBlue => {
        fetchFile(req.body.xsdGUri, req.headers.authorization).then(
          xsdGreen => {
            const xmlIsWellformed = libxml.parseXml(xml)
            const xsdBlueDoc = libxml.parseXml(xsdBlue, {
              baseUrl: 'app/assets/xsl/xsd/publishing/',
            })
            const xsdGreenDoc = libxml.parseXml(xsdGreen, {
              baseUrl: 'app/assets/xsl/xsd/archiving/',
            })
            const xmlIsValid = xmlIsWellformed.validate(xsdBlueDoc)
            const errors = xmlIsWellformed.validationErrors
            if (!xmlIsValid) {
              let errString = 'Invalid XML: <br/><br/>'
              errors.forEach((err, i) => {
                errString += `${err}<br/>Line: ${err.line}.`
                if (err.str1) {
                  errString += ` ID: ${err.str1}.`
                }
                if (i !== errors.length - 1) {
                  errString += `<br/><br/>`
                }
              })
              res.status(400).send(errString)
            } else {
              transformXML(xml, 'app/assets/xsl/pnihms2pmc3.xsl')
                .then(nxml => {
                  const nxmlIsWellformed = libxml.parseXml(nxml)
                  const nxmlIsValid = nxmlIsWellformed.validate(xsdGreenDoc)
                  const nxmlErrors = nxmlIsWellformed.validationErrors
                  if (!nxmlIsValid) {
                    let errString = 'Invalid NXML: <br/><br/>'
                    nxmlErrors.forEach((err, i) => {
                      errString += `${err}<br/>Line: ${err.line}.`
                      if (err.str1) {
                        errString += ` ID: ${err.str1}.`
                      }
                      if (i !== nxmlErrors.length - 1) {
                        errString += `<br/><br/>`
                      }
                    })
                    res.status(400).send(errString)
                  } else {
                    // Check NXML against the stylechecker
                    transformXML(
                      nxml,
                      'app/assets/xsl/stylechecker/nlm-stylechecker.xsl',
                    ).then(checked => {
                      const result = libxml.parseXml(checked)
                      const styleErrors = result.find('//error')
                      if (styleErrors.length === 0) {
                        const { files } = req.body.currentVersion
                        let filelist
                        if (files) {
                          filelist = getFilelist(files)
                        }
                        // now we will transform the nXML into HTML:
                        transformXML(
                          nxml,
                          'app/assets/xsl/nxml2html.xsl',
                          filelist,
                        ).then(html => {
                          addXMLFiles(
                            nxml,
                            html,
                            req.body.project,
                            req.body.currentVersion,
                            req.headers.authorization,
                          ).then(resp => {
                            // send back new filelist
                            res.send(resp)
                          })
                        })
                      } else {
                        let styleErrString = `Style Errors: <br/><br/>`
                        styleErrors.forEach((err, i) => {
                          styleErrString += err.text()
                          if (i !== styleErrors.length - 1) {
                            styleErrString += `<br/><br/>`
                          }
                        })
                        res.status(400).send(styleErrString)
                      }
                    })
                  }
                })
                .catch(err => {
                  res.status(400).send(JSON.stringify(err.message))
                })
            }
          },
        )
      })
    })
  })
}
