const { Model } = require('objection')
const uuid = require('uuid')
const EpmcBaseModel = require('../epmc-base-model')
const { rowToEntity, entityToRow, buildQuery, runQuery } = require('../util')

const columnNames = ['quote', 'comment', 'ranges', 'user_id', 'review_id']

class Annotation extends EpmcBaseModel {
  static get tableName() {
    return 'annotation'
  }

  static get schema() {
    return {
      properties: {
        id: { type: 'uuid' },
        created: { type: 'timestamp' },
        updated: { type: 'timestamp' },
        deleted: { type: 'timestamp' },
        userId: { type: 'uuid' },
        reviewId: { type: 'uuid' },
        quote: { type: 'string' },
        comment: { type: 'string' },
        ranges: { type: 'json' },
      },
    }
  }

  static get relationMappings() {
    const User = require('../user/data-access')
    const Review = require('../review/data-access')
    return {
      user: {
        relation: Model.BelongsToOneRelation,
        modelClass: User,
        join: {
          from: 'annotation.userId',
          to: 'users.id',
        },
      },
      review: {
        relation: Model.BelongsToOneRelation,
        modelClass: Review,
        join: {
          from: 'annotation.reviewId',
          to: 'review.id',
        },
      },
    }
  }

  static async selectById(id) {
    const rows = await runQuery(
      buildQuery
        .select()
        .from('annotation')
        .where({ id })
        .whereNull('deleted'),
    )
    if (!rows.length) {
      throw new Error('Annotation not found')
    }
    return rowToEntity(rows[0])
  }

  static async selectByReviewId(reviewId) {
    const rows = await runQuery(
      buildQuery
        .select()
        .from('annotation')
        .where({ 'annotation.review_id': reviewId })
        .whereNull('annotation.deleted'),
    )
    return rows.map(rowToEntity)
  }

  static async selectAll() {
    const rows = await runQuery(
      buildQuery
        .select()
        .from('annotation')
        .whereNull('deleted'),
    )
    return rows.map(rowToEntity)
  }

  static async insert(annotation) {
    const row = entityToRow(annotation, columnNames)
    row.id = uuid.v4()
    const query = buildQuery.insert(row).into('annotation')
    await runQuery(query)
    return row.id
  }

  static update(annotation) {
    const row = entityToRow(annotation, columnNames)
    const query = buildQuery
      .update(row)
      .table('annotation')
      .where('id', annotation.id)
      .whereNull('deleted')
    return runQuery(query)
  }

  static delete(id) {
    return Annotation.query().updateAndFetchById(id, {
      deleted: new Date().toISOString(),
    })
  }

  static deleteByReviewId(review_id) {
    return Annotation.query()
      .update({
        deleted: new Date().toISOString(),
      })
      .where({ review_id })
      .whereNull('deleted')
  }
}
module.exports = Annotation
